@@ -1,9 +1,9 @@
-## dist.topo.R (2021-04-18)
+## dist.topo.R (2022-03-01)
 
 ##      Topological Distances, Tree Bipartitions,
 ##   Consensus Trees, and Bootstrapping Phylogenies
 
-## Copyright 2005-2021 Emmanuel Paradis, 2016-2021 Klaus Schliep
+## Copyright 2005-2022 Emmanuel Paradis, 2016-2021 Klaus Schliep
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -380,7 +380,7 @@ SHORTwise <- function(x) {
     x
 }
 
-consensus <- function(..., p = 1, check.labels = TRUE)
+consensus <- function(..., p = 1, check.labels = TRUE, rooted = FALSE)
 {
     foo <- function(ic, node) {
         ## ic: index of 'pp'
@@ -415,15 +415,20 @@ consensus <- function(..., p = 1, check.labels = TRUE)
         labels <- obj[[1]]$tip.label
         if (check.labels) obj <- .compressTipLabel(obj)
     }
+    if(!rooted) obj <- root(obj, 1)
+
     ntree <- length(obj)
     ## Get all observed partitions and their frequencies:
     pp <- prop.part(obj, check.labels = FALSE)
     ## Drop the partitions whose frequency is less than 'p':
     if (p == 0.5) p <- 0.5000001 # avoid incompatible splits
-    pp <- pp[attr(pp, "number") >= p * ntree]
+    bs <- attr(pp, "number")
+    pp <- pp[bs >= p * ntree]
+    bs <- bs[bs >= p * ntree]
     ## Get the order of the remaining partitions by decreasing size:
     ind <- order(lengths(pp), decreasing = TRUE)
     pp <- pp[ind]
+    bs <- bs[ind]
     n <- length(labels)
     m <- length(pp)
     edge <- matrix(0L, n + m - 1, 2)
@@ -435,6 +440,6 @@ consensus <- function(..., p = 1, check.labels = TRUE)
         pos <- 1L
         foo(1, nextnode)
     }
-    structure(list(edge = edge, tip.label = labels,
+    structure(list(edge = edge, tip.label = labels, node.label = bs/ntree,
               Nnode = m), class = "phylo")
 }
